@@ -35,14 +35,20 @@ Tile caching (creation, caching and retrieval of tiles).
 
 """
 
-from __future__ import with_statement
 
+from functools import partial
 from contextlib import contextmanager
 from mapproxy.grid import MetaGrid
+from mapproxy.image import BlankImageSource
+from mapproxy.image.opts import ImageOptions
 from mapproxy.image.merge import merge_images
-from mapproxy.image.tile import TileSplitter
+from mapproxy.image.tile import TileSplitter, TiledImage
 from mapproxy.layer import MapQuery, BlankImage
-from mapproxy.util import async
+from mapproxy.source import SourceError
+from mapproxy.util import async_
+from mapproxy.util.py import reraise, reraise_exception
+import sys
+
 
 class TileManager(object):
     """
@@ -55,8 +61,12 @@ class TileManager(object):
         return this or a new tile object.
     """
     def __init__(self, grid, cache, sources, format, locker, image_opts=None, request_format=None,
-        meta_buffer=None, meta_size=None, minimize_meta_requests=False, identifier=None,
-        pre_store_filter=None, concurrent_tile_creators=1, tile_creator_class=None):
+            meta_buffer=None, meta_size=None, minimize_meta_requests=False, identifier=None,
+            pre_store_filter=None, concurrent_tile_creators=1, tile_creator_class=None,
+            bulk_meta_tiles=False,
+            rescale_tiles=0,
+            cache_rescaled_tiles=False,
+        ):
         self.grid = grid
         self.cache = cache
         self.locker = locker
@@ -68,16 +78,24 @@ class TileManager(object):
         self.sources = sources
         self.minimize_meta_requests = minimize_meta_requests
         self._expire_timestamp = None
-        self.transparent = self.sources[0].transparent
+        self._refresh_before = {}
         self.pre_store_filter = pre_store_filter or []
         self.concurrent_tile_creators = concurrent_tile_creators
         self.tile_creator_class = tile_creator_class or TileCreator
 
+        self.rescale_tiles = rescale_tiles
+        self.cache_rescaled_tiles = cache_rescaled_tiles
+
         if meta_buffer or (meta_size and not meta_size == [1, 1]):
             if all(source.supports_meta_tiles for source in sources):
                 self.meta_grid = MetaGrid(grid, meta_size=meta_size, meta_buffer=meta_buffer)
             elif any(source.supports_meta_tiles for source in sources):
                 raise ValueError('meta tiling configured but not supported by all sources')
+            elif meta_size and not meta_size == [1, 1] and bulk_meta_tiles:
+                # meta tiles configured but all sources are tiled
+                # use bulk_meta_tile mode that download tiles in parallel
+                self.meta_grid = MetaGrid(grid, meta_size=meta_size, meta_buffer=0)
+                self.tile_creator_class = partial(self.tile_creator_class, bulk_meta_tiles=True)
 
     @contextmanager
     def session(self):
@@ -97,23 +115,52 @@ class TileManager(object):
             self.cache.cleanup()
 
     def load_tile_coord(self, tile_coord, dimensions=None, with_metadata=False):
-        tile = Tile(tile_coord)
-        self.cache.load_tile(tile, with_metadata)
+        return self.load_tile_coords(
+            [tile_coord], dimensions=dimensions, with_metadata=with_metadata,
+        )[0]
 
-        if tile.coord is not None and not self.is_cached(tile, dimensions=dimensions):
-            # missing or staled
-            creator = self.creator(dimensions=dimensions)
-            created_tiles = creator.create_tiles([tile])
-            for created_tile in created_tiles:
-                if created_tile.coord == tile_coord:
-                    return created_tile
-
-        return tile
 
     def load_tile_coords(self, tile_coords, dimensions=None, with_metadata=False):
         tiles = TileCollection(tile_coords)
+        rescale_till_zoom = 0
+        if self.rescale_tiles:
+            rescaled_tiles = {}
+
+            for t in tiles.tiles:
+                # Use zoom level from first None tile.
+                if t.coord is not None:
+                    rescale_till_zoom = t.coord[2] + self.rescale_tiles
+                    break
+            else:
+                return tiles
+
+            if rescale_till_zoom < 0:
+                rescale_till_zoom = 0
+            if rescale_till_zoom > self.grid.levels:
+                rescale_till_zoom = self.grid.levels
+
+        tiles = self._load_tile_coords(
+            tiles, dimensions=dimensions, with_metadata=with_metadata,
+            rescale_till_zoom=rescale_till_zoom, rescaled_tiles={},
+        )
+
+        for t in tiles.tiles:
+            # Remove our internal marker source, for missing tiles.
+            if t.source is RESCALE_TILE_MISSING:
+                t.source = None
+
+        return tiles
+
+    def _load_tile_coords(self, tiles, dimensions=None, with_metadata=False,
+                          rescale_till_zoom=None, rescaled_tiles=None,
+        ):
         uncached_tiles = []
 
+        if rescaled_tiles:
+            for t in tiles:
+                if t.coord in rescaled_tiles:
+                    t.source = rescaled_tiles[t.coord].source
+
         # load all in batch
         self.cache.load_tiles(tiles, with_metadata)
 
@@ -125,6 +172,9 @@ class TileManager(object):
         if uncached_tiles:
             creator = self.creator(dimensions=dimensions)
             created_tiles = creator.create_tiles(uncached_tiles)
+            if not created_tiles and self.rescale_tiles:
+                created_tiles = [self._scaled_tile(t, rescale_till_zoom, rescaled_tiles) for t in uncached_tiles]
+
             for created_tile in created_tiles:
                 if created_tile.coord in tiles:
                     tiles[created_tile.coord].source = created_tile.source
@@ -155,7 +205,9 @@ class TileManager(object):
         max_mtime = self.expire_timestamp(tile)
         if cached and max_mtime is not None:
             self.cache.load_tile_metadata(tile)
-            stale = tile.timestamp < max_mtime
+            # file time stamp must be rounded to integer since time conversion functions
+            # mktime and timetuple strip decimals from seconds
+            stale = int(tile.timestamp) <= max_mtime
             if stale:
                 cached = False
         return cached
@@ -181,6 +233,9 @@ class TileManager(object):
 
         :note: Returns _expire_timestamp by default.
         """
+        if self._refresh_before:
+            from mapproxy.seed.config import before_timestamp_from_options
+            return before_timestamp_from_options(self._refresh_before)
         return self._expire_timestamp
 
     def apply_tile_filter(self, tile):
@@ -195,14 +250,75 @@ class TileManager(object):
             tile = img_filter(tile)
         return tile
 
+    def _scaled_tile(self, tile, stop_zoom, rescaled_tiles):
+        """
+        Try to load tile by loading, scaling and clipping tiles from zoom levels above or
+        below. stop_zoom determines if tiles from above should be scaled up, or if tiles
+        from below should be scaled down.
+        Returns an empty Tile if tile zoom level is stop_zoom.
+        """
+        if tile.coord in rescaled_tiles:
+            return rescaled_tiles[tile.coord]
+
+        # Cache tile in rescaled_tiles. We initially set source to a fixed
+        # BlankImageSource and overwrite it if we actually rescaled the tile.
+        tile.source = RESCALE_TILE_MISSING
+        rescaled_tiles[tile.coord] = tile
+
+        tile_bbox = self.grid.tile_bbox(tile.coord)
+        current_zoom = tile.coord[2]
+        if stop_zoom == current_zoom:
+            return tile
+        if stop_zoom > current_zoom:
+            src_level = current_zoom + 1
+        else:
+            src_level = current_zoom - 1
+
+        src_bbox, src_tile_grid, affected_tile_coords = self.grid.get_affected_level_tiles(tile_bbox, src_level)
+
+        affected_tiles = TileCollection(affected_tile_coords)
+        for t in affected_tiles:
+            # Add sources of cached tiles, to avoid loading same tile multiple times
+            # loading recursive.
+            if t.coord in rescaled_tiles:
+                t.source = rescaled_tiles[t.coord].source
+
+        tile_collection = self._load_tile_coords(
+            affected_tiles,
+            rescale_till_zoom=stop_zoom,
+            rescaled_tiles=rescaled_tiles,
+        )
+
+        if tile_collection.blank:
+            return tile
+
+        tile_sources = []
+        for t in tile_collection:
+            # Replace RESCALE_TILE_MISSING with None, before transforming tiles.
+            tile_sources.append(t.source if t.source is not RESCALE_TILE_MISSING else None)
+
+        tiled_image = TiledImage(tile_sources, src_bbox=src_bbox, src_srs=self.grid.srs,
+                            tile_grid=src_tile_grid, tile_size=self.grid.tile_size)
+        tile.source = tiled_image.transform(tile_bbox, self.grid.srs, self.grid.tile_size, self.image_opts)
+
+        if self.cache_rescaled_tiles:
+            self.cache.store_tile(tile)
+        return tile
+
+# RESCALE_TILE_MISSING is a dummy source to prevent a tile cache from loading
+# a tile that we already found out is missing.
+RESCALE_TILE_MISSING = BlankImageSource((256, 256), ImageOptions())
+
 class TileCreator(object):
-    def __init__(self, tile_mgr, dimensions=None):
+    def __init__(self, tile_mgr, dimensions=None, image_merger=None, bulk_meta_tiles=False):
         self.cache = tile_mgr.cache
         self.sources = tile_mgr.sources
         self.grid = tile_mgr.grid
         self.meta_grid = tile_mgr.meta_grid
+        self.bulk_meta_tiles = bulk_meta_tiles
         self.tile_mgr = tile_mgr
         self.dimensions = dimensions
+        self.image_merger = image_merger
 
     def is_cached(self, tile):
         """
@@ -210,7 +326,15 @@ class TileCreator(object):
         """
         return self.tile_mgr.is_cached(tile)
 
+    def is_stale(self, tile):
+        """
+        Return True if the tile exists in cache and is expired.
+        """
+        return self.tile_mgr.is_stale(tile)
+
     def create_tiles(self, tiles):
+        if not self.sources:
+            return []
         if not self.meta_grid:
             created_tiles = self._create_single_tiles(tiles)
         elif self.tile_mgr.minimize_meta_requests and len(tiles) > 1:
@@ -241,7 +365,7 @@ class TileCreator(object):
 
     def _create_threaded(self, create_func, tiles):
         result = []
-        async_pool = async.Pool(self.tile_mgr.concurrent_tile_creators)
+        async_pool = async_.Pool(self.tile_mgr.concurrent_tile_creators)
         for new_tiles in async_pool.imap(create_func, tiles):
             result.extend(new_tiles)
         return result
@@ -252,10 +376,23 @@ class TileCreator(object):
                          self.tile_mgr.request_format, dimensions=self.dimensions)
         with self.tile_mgr.lock(tile):
             if not self.is_cached(tile):
-                source = self._query_sources(query)
+                try:
+                    source = self._query_sources(query)
+                # if source is not available, try to serve tile in cache
+                except SourceError as e:
+                    if self.is_stale(tile):
+                        self.cache.load_tile(tile)
+                    else:
+                        reraise_exception(e, sys.exc_info())
                 if not source: return []
-                # call as_buffer to force conversion into cache format
-                source.as_buffer(self.tile_mgr.image_opts)
+                if source.authorize_stale and self.is_stale(tile):
+                    # The configuration authorises blank tiles generated by the error_handler
+                    # to be replaced by stale tiles from cache.
+                    self.cache.load_tile(tile)
+                    return [tile]
+                if self.tile_mgr.image_opts != source.image_opts:
+                    # call as_buffer to force conversion into cache format
+                    source.as_buffer(self.tile_mgr.image_opts)
                 source.image_opts = self.tile_mgr.image_opts
                 tile.source = source
                 tile.cacheable = source.cacheable
@@ -271,7 +408,14 @@ class TileCreator(object):
         Query all sources and return the results as a single ImageSource.
         Multiple sources will be merged into a single image.
         """
-        if len(self.sources) == 1:
+
+        # directly return get_map without merge if ...
+        if (len(self.sources) == 1 and
+            not self.image_merger and # no special image_merger (like BandMerger)
+            not (self.sources[0].coverage and  # no clipping coverage
+                 self.sources[0].coverage.clip and
+                 self.sources[0].coverage.intersects(query.bbox, query.srs))
+        ):
             try:
                 return self.sources[0].get_map(query)
             except BlankImage:
@@ -281,19 +425,25 @@ class TileCreator(object):
             try:
                 img = source.get_map(query)
             except BlankImage:
-                return None
+                return None, None
             else:
-                return img
+                return (img, source.coverage)
 
-        imgs = []
-        for img in async.imap(get_map_from_source, self.sources):
-            if img is not None:
-                imgs.append(img)
+        layers = []
+        for layer in async_.imap(get_map_from_source, self.sources):
+            if layer[0] is not None:
+                layers.append(layer)
 
-        if not imgs: return None
-        return merge_images(imgs, size=query.size, image_opts=self.tile_mgr.image_opts)
+        return merge_images(layers, size=query.size, bbox=query.bbox, bbox_srs=query.srs,
+                            image_opts=self.tile_mgr.image_opts, merger=self.image_merger)
 
     def _create_meta_tiles(self, meta_tiles):
+        if self.bulk_meta_tiles:
+            created_tiles = []
+            for meta_tile in meta_tiles:
+                    created_tiles.extend(self._create_bulk_meta_tile(meta_tile))
+            return created_tiles
+
         if self.tile_mgr.concurrent_tile_creators > 1 and len(meta_tiles) > 1:
             return self._create_threaded(self._create_meta_tile, meta_tiles)
 
@@ -303,6 +453,10 @@ class TileCreator(object):
         return created_tiles
 
     def _create_meta_tile(self, meta_tile):
+        """
+        _create_meta_tile queries a single meta tile and splits it into
+        tiles.
+        """
         tile_size = self.grid.tile_size
         query = MapQuery(meta_tile.bbox, meta_tile.size, self.grid.srs, self.tile_mgr.request_format,
             dimensions=self.dimensions)
@@ -317,11 +471,64 @@ class TileCreator(object):
                 if meta_tile_image.cacheable:
                     self.cache.store_tiles(splitted_tiles)
                 return splitted_tiles
-        # else
+            # else
         tiles = [Tile(coord) for coord in meta_tile.tiles]
         self.cache.load_tiles(tiles)
         return tiles
 
+    def _create_bulk_meta_tile(self, meta_tile):
+        """
+        _create_bulk_meta_tile queries each tile of the meta tile in parallel
+        (using concurrent_tile_creators).
+        """
+        tile_size = self.grid.tile_size
+        main_tile = Tile(meta_tile.main_tile_coord)
+        with self.tile_mgr.lock(main_tile):
+            if not all(self.is_cached(t) for t in meta_tile.tiles if t is not None):
+                async_pool = async_.Pool(self.tile_mgr.concurrent_tile_creators)
+                def query_tile(coord):
+                    try:
+                        query = MapQuery(self.grid.tile_bbox(coord), tile_size, self.grid.srs, self.tile_mgr.request_format,
+                            dimensions=self.dimensions)
+                        tile_image = self._query_sources(query)
+                        if tile_image is None:
+                            return None
+
+                        if self.tile_mgr.image_opts != tile_image.image_opts:
+                            # call as_buffer to force conversion into cache format
+                            tile_image.as_buffer(self.tile_mgr.image_opts)
+
+                        tile = Tile(coord, cacheable=tile_image.cacheable)
+                        tile.source = tile_image
+                        tile = self.tile_mgr.apply_tile_filter(tile)
+                    except BlankImage:
+                        return None
+                    else:
+                        return tile
+
+                tiles = []
+                for tile_task in async_pool.imap(query_tile,
+                    [t for t in meta_tile.tiles if t is not None],
+                    use_result_objects=True,
+                ):
+                    if tile_task.exception is None:
+                        tile = tile_task.result
+                        if tile is not None:
+                            tiles.append(tile)
+                    else:
+                        ex = tile_task.exception
+                        async_pool.shutdown(True)
+                        reraise(ex)
+
+                self.cache.store_tiles([t for t in tiles if t.cacheable])
+                return tiles
+
+            # else
+        tiles = [Tile(coord) for coord in meta_tile.tiles]
+        self.cache.load_tiles(tiles)
+        return tiles
+
+
 class Tile(object):
     """
     Internal data object for all tiles. Stores the tile-``coord`` and the tile data.
@@ -458,6 +665,13 @@ class TileCollection(object):
         """
         return all((t.source is None for t in self.tiles))
 
+    @property
+    def blank(self):
+        """
+        Returns True if all sources collection are BlankImageSources or have not source at all.
+        """
+        return all((t.source is None or isinstance(t.source, BlankImageSource) for t in self.tiles))
+
     def __repr__(self):
         return 'TileCollection(%r)' % self.tiles
 
