@@ -38,4 +38,9 @@ if PY2:
     except ImportError:
         from StringIO import StringIO as BytesIO
 else:
-    from io import BytesIO
\ No newline at end of file
+    from io import BytesIO
+
+if PY2:
+    raw_input = raw_input
+else:
+    raw_input = input
