@@ -293,7 +293,7 @@ class GridConfiguration(ConfigurationBase):
             global_key='image.max_shrink_factor')
 
         if conf.get('origin') is None:
-            log.warn('grid %s does not have an origin. default origin will change from sw (south/west) to nw (north-west) with MapProxy 2.0',
+            log.warning('grid %s does not have an origin. default origin will change from sw (south/west) to nw (north-west) with MapProxy 2.0',
                 conf['name'],
             )
 
@@ -591,10 +591,13 @@ class SourceConfiguration(ConfigurationBase):
 
         timeout = self.context.globals.get_value('http.client_timeout', self.conf)
         headers = self.context.globals.get_value('http.headers', self.conf)
+        hide_error_details = self.context.globals.get_value('http.hide_error_details', self.conf)
+        manage_cookies = self.context.globals.get_value('http.manage_cookies', self.conf)
 
         http_client = HTTPClient(url, username, password, insecure=insecure,
                                  ssl_ca_certs=ssl_ca_certs, timeout=timeout,
-                                 headers=headers)
+                                 headers=headers, hide_error_details=hide_error_details,
+                                 manage_cookies=manage_cookies)
         return http_client, url
 
     @memoize
@@ -608,11 +611,12 @@ class SourceConfiguration(ConfigurationBase):
                 raise ConfigurationError("invalid error code %r in on_error", status_code)
             cacheable = response_conf.get('cache', False)
             color = response_conf.get('response', 'transparent')
+            authorize_stale = response_conf.get('authorize_stale', False)
             if color == 'transparent':
                 color = (255, 255, 255, 0)
             else:
                 color = parse_color(color)
-            error_handler.add_handler(status_code, color, cacheable)
+            error_handler.add_handler(status_code, color, cacheable, authorize_stale)
 
         return error_handler
 
@@ -974,7 +978,7 @@ class TileSourceConfiguration(SourceConfiguration):
 
         grid_name = self.conf.get('grid')
         if grid_name is None:
-            log.warn("tile source for %s does not have a grid configured and defaults to GLOBAL_MERCATOR. default will change with MapProxy 2.0", url)
+            log.warning("tile source for %s does not have a grid configured and defaults to GLOBAL_MERCATOR. default will change with MapProxy 2.0", url)
             grid_name = "GLOBAL_MERCATOR"
 
         grid = self.context.grids[grid_name].tile_grid()
@@ -1022,7 +1026,7 @@ class CacheConfiguration(ConfigurationBase):
         cache_dir = self.conf.get('cache', {}).get('directory')
         if cache_dir:
             if self.conf.get('cache_dir'):
-                log.warn('found cache.directory and cache_dir option for %s, ignoring cache_dir',
+                log.warning('found cache.directory and cache_dir option for %s, ignoring cache_dir',
                 self.conf['name'])
             return self.context.globals.abspath(cache_dir)
 
@@ -1060,7 +1064,7 @@ class CacheConfiguration(ConfigurationBase):
             global_key='cache.link_single_color_images')
 
         if link_single_color_images and sys.platform == 'win32':
-            log.warn('link_single_color_images not supported on windows')
+            log.warning('link_single_color_images not supported on windows')
             link_single_color_images = False
 
         return FileCache(
@@ -1451,8 +1455,8 @@ class CacheConfiguration(ConfigurationBase):
         from mapproxy.layer import map_extent_from_grid, merge_layer_extents
 
         base_image_opts = self.image_opts()
-        if self.conf.get('format') == 'mixed' and not self.conf.get('request_format') == 'image/png':
-            raise ConfigurationError('request_format must be set to image/png if mixed mode is enabled')
+        if self.conf.get('format') == 'mixed' and not self.conf.get('request_format') in [ 'image/png', 'image/vnd.jpeg-png' ]:
+            raise ConfigurationError('request_format must be set to image/png or image/vnd.jpeg-png if mixed mode is enabled')
         request_format = self.conf.get('request_format') or self.conf.get('format')
         if '/' in request_format:
             request_format_ext = request_format.split('/', 1)[1]
@@ -1571,6 +1575,8 @@ class CacheConfiguration(ConfigurationBase):
                 cache_rescaled_tiles=cache_rescaled_tiles,
                 rescale_tiles=rescale_tiles,
             )
+            if self.conf['name'] in self.context.caches:
+                mgr._refresh_before = self.context.caches[self.conf['name']].conf.get('refresh_before', {})
             extent = merge_layer_extents(sources)
             if extent.is_default:
                 extent = map_extent_from_grid(tile_grid)
@@ -1581,7 +1587,7 @@ class CacheConfiguration(ConfigurationBase):
     def grid_confs(self):
         grid_names = self.conf.get('grids')
         if grid_names is None:
-            log.warn('cache %s does not have any grids. default will change from [GLOBAL_MERCATOR] to [GLOBAL_WEBMERCATOR] with MapProxy 2.0',
+            log.warning('cache %s does not have any grids. default will change from [GLOBAL_MERCATOR] to [GLOBAL_WEBMERCATOR] with MapProxy 2.0',
                 self.conf['name'])
             grid_names = ['GLOBAL_MERCATOR']
         return [(g, self.context.grids[g]) for g in grid_names]
@@ -1936,7 +1942,7 @@ class ServiceConfiguration(ConfigurationBase):
             fi_template = conf.get('restful_featureinfo_template')
             if template and '{{' in template:
                 # TODO remove warning in 1.6
-                log.warn("double braces in WMTS restful_template are deprecated {{x}} -> {x}")
+                log.warning("double braces in WMTS restful_template are deprecated {{x}} -> {x}")
             services.append(
                 WMTSRestServer(
                     layers, md, template=template,
@@ -1974,7 +1980,7 @@ class ServiceConfiguration(ConfigurationBase):
         for format in image_formats_names:
             opts = self.context.globals.image_options.image_opts({}, format)
             if opts.format in image_formats:
-                log.warn('duplicate mime-type for WMS image_formats: "%s" already configured, will use last format',
+                log.warning('duplicate mime-type for WMS image_formats: "%s" already configured, will use last format',
                     opts.format)
             image_formats[opts.format] = opts
         info_types = conf.get('featureinfo_types')
@@ -2071,13 +2077,13 @@ def load_configuration(mapproxy_conf, seed=False, ignore_warnings=True, renderd=
 
     errors, informal_only = validate_options(conf_dict)
     for error in errors:
-        log.warn(error)
+        log.warning(error)
     if not informal_only or (errors and not ignore_warnings):
         raise ConfigurationError('invalid configuration')
 
     errors = validate_references(conf_dict)
     for error in errors:
-        log.warn(error)
+        log.warning(error)
 
     return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir, seed=seed,
         renderd=renderd)
@@ -2149,5 +2155,3 @@ def parse_color(color):
         return r, g, b, a
 
     return r, g, b
-
-
