@@ -16,15 +16,9 @@
 """
 Service requests (parsing, handling, etc).
 """
-import cgi
-
+from mapproxy.compat import PY2, iteritems, text_type
+from mapproxy.compat.modules import parse_qsl, urlparse, quote
 from mapproxy.util.py import cached_property
-from mapproxy.compat import iteritems, PY2, text_type
-
-if PY2:
-    from urllib import quote
-else:
-    from urllib.parse import quote
 
 class NoCaseMultiDict(dict):
     """
@@ -178,7 +172,7 @@ def url_decode(qs, charset='utf-8', decode_keys=False, include_empty=True,
     Parse query string `qs` and return a `NoCaseMultiDict`.
     """
     tmp = []
-    for key, value in cgi.parse_qsl(qs, include_empty):
+    for key, value in parse_qsl(qs, include_empty):
         if PY2:
             if decode_keys:
                 key = key.decode(charset, errors)
@@ -265,12 +259,27 @@ class Request(object):
     def host_url(self):
         return '%s://%s/' % (self.url_scheme, self.host)
 
+    @cached_property
+    def server_url(self):
+        return 'http://%s:%s/' % (
+            self.environ['SERVER_NAME'],
+            self.environ['SERVER_PORT']
+        )
+
     @property
     def script_url(self):
         "Full script URL without trailing /"
         return (self.host_url.rstrip('/') +
                 quote(self.environ.get('SCRIPT_NAME', '/').rstrip('/'))
                )
+    
+    @property
+    def server_script_url(self):
+        "Internal script URL"
+        return self.script_url.replace(
+            self.host_url.rstrip('/'),
+            self.server_url.rstrip('/')
+        )
 
     @property
     def base_url(self):
