@@ -16,7 +16,6 @@
 """
 Service requests (parsing, handling, etc).
 """
-from __future__ import with_statement
 import codecs
 from mapproxy.request.wms import exception
 from mapproxy.exception import RequestError
@@ -73,8 +72,8 @@ class WMSMapRequestParams(RequestParams):
         """
         if 'height' not in self or 'width' not in self:
             return None
-        width = int(self.params['width'])
-        height = int(self.params['height'])
+        width = int(float(self.params['width'])) # allow float sizes (100.0), but truncate decimals
+        height = int(float(self.params['height']))
         return (width, height)
     def _set_size(self, value):
         self['width'] = str(value[0])
@@ -230,7 +229,7 @@ class WMSMapRequest(WMSRequest):
     def validate_styles(self):
         if 'styles' in self.params:
             styles = self.params['styles']
-            if not set(styles.split(',')).issubset(set(['default', ''])):
+            if not set(styles.split(',')).issubset(set(['default', '', 'inspire_common:DEFAULT'])):
                 raise RequestError('unsupported styles: ' + self.params['styles'],
                                    code='StyleNotDefined', request=self)
 
@@ -290,6 +289,10 @@ class WMS100MapRequest(WMSMapRequest):
         params = WMSMapRequest.adapt_params_to_version(self)
         del params['version']
         del params['service']
+
+        image_format = params['format']
+        if '/' in image_format:
+            params['format'] = image_format.split('/', 1)[1].upper()
         return params
 
     def validate_format(self, image_formats):
@@ -328,7 +331,7 @@ def switch_bbox_epsg_axis_order(bbox, srs):
             if SRS(srs).is_axis_order_ne:
                 return bbox[1], bbox[0], bbox[3], bbox[2]
         except RuntimeError:
-            log.warn('unknown SRS %s' % srs)
+            log.warning('unknown SRS %s' % srs)
     return bbox
 
 def _switch_bbox(self):
@@ -423,10 +426,13 @@ class WMSFeatureInfoRequestParams(WMSMapRequestParams):
 
     def _get_pos(self):
         """x, y query image coordinates (in pixel)"""
+        if '.' in self['x'] or '.' in self['y']:
+            return float(self['x']), float(self['y'])
         return int(self['x']), int(self['y'])
+
     def _set_pos(self, value):
-        self['x'] = str(int(round(value[0])))
-        self['y'] = str(int(round(value[1])))
+        self['x'] = str(value[0])
+        self['y'] = str(value[1])
     pos = property(_get_pos, _set_pos)
     del _get_pos
     del _set_pos
@@ -626,7 +632,7 @@ def _parse_version(req):
     if 'wmtver' in req.args:
         return Version(req.args['wmtver'])
 
-    return Version('1.1.1') # default
+    return None
 
 def _parse_request_type(req):
     if 'request' in req.args:
@@ -678,7 +684,16 @@ def wms_request(req, validate=True, strict=True, versions=None):
     version = _parse_version(req)
     req_type = _parse_request_type(req)
 
-    if versions and version not in versions:
+    if versions is None:
+        versions = sorted([
+            Version(v) for v in ('1.0.0', '1.1.0', '1.1.1', '1.3.0')])
+
+    if version is None:
+        # If no version number is specified in the request,
+        # the server shall respond with the highest version.
+        version = max(versions)
+
+    if version not in versions:
         version_requests = None
     else:
         version_requests = request_mapping.get(version, None)
@@ -727,10 +742,12 @@ info_formats = {
     Version('1.3.0'): (('text', 'text/plain'),
                        ('html', 'text/html'),
                        ('xml', 'text/xml'),
+                       ('json', 'application/json'),
                       ),
     None: (('text', 'text/plain'),
            ('html', 'text/html'),
            ('xml', 'application/vnd.ogc.gml'),
+           ('json', 'application/json'),
           )
 }
 
